<?php

namespace Drupal\content_packager;

use Drupal\views\Views;

/**
 * Batch operations to apply to Views for content packaging.
 *
 * @package content_packager
 */
class BatchOperations {

  /**
   * Implements callback_batch_operation().
   *
   * @param string $packageUri
   *   The view rows to process.
   * @param array|\ArrayAccess $context
   *   The batch context array.
   */
  public static function prepareDestination($packageUri, &$context) {
    if (!isset($context['results']['prepared'])) {
      $context['results']['prepared'] = FALSE;
    }

    // We really shouldn't be doing something destructive like a recursive
    // delete if we're encounting any problems.
    if (!file_valid_uri($packageUri) || !file_unmanaged_prepare($packageUri)) {
      $msg = t('Packager directory might not be configured correctly.');
      \Drupal::messenger()->addError($msg);
      return;
    }

    if (!file_unmanaged_delete_recursive($packageUri)) {
      $msg = t('Packager directory could not be deleted.');
      \Drupal::messenger()->addError($msg);
    }

    if (!file_prepare_directory($packageUri, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
      $msg = t('Packager directory could not be prepared.');
      \Drupal::messenger()->addError($msg);
      return;
    }

    $context['results']['prepared'] = TRUE;
  }

  /**
   * Implements callback_batch_operation().
   *
   * First renders a view and stores its rendered output onto the filesystem.
   *
   * @param string $view_id
   *   A view to render.
   * @param string $display_id
   *   The specific display ID to render on the view.
   * @param string $packageUri
   *   A URI destination.
   * @param array|\ArrayAccess $context
   *   The batch context array.
   */
  public static function renderAndSaveViewOutput($view_id, $display_id, $data_uri, &$context) {

    if (!isset($context['results']['rendered'])) {
      $context['results']['rendered'] = FALSE;
      $context['results']['failed'] = [];
      $context['results']['completed_count'] = 0;
    }

    $view = Views::getView($view_id);
    $view->setDisplay($display_id);
    $render_array = $view->preview();

    $context['results']['rendered'] = TRUE;

    // Must be time to save output!
    $output = $render_array['#markup'];

    if (!isset($context['results']['prepared']) || $context['results']['prepared'] !== TRUE) {
      $context['results']['failed']['copy'] = ['id' => $data_uri, 'type' => 'data export'];
      return;
    }

    if (!file_put_contents($data_uri, $output)) {
      $context['results']['failed']['copy'] = ['id' => $data_uri, 'type' => 'data export'];
      $msg = t('Failed to output the data to file!');

      \Drupal::messenger()->addError($msg);
    }

    $context['results']['completed_count']++;
  }

  public static function copyEntityFilesAndZip(array $entity_infos, $package_uri, $options, &$context) {
    if (!isset($context['results']['failed']['copy'])) {
      $context['results']['failed']['copy'] = [];
      $context['results']['failed']['zip'] = [];
    }

    $uris = [];
    foreach ($entity_infos as $info) {
      $id = $info['id'];
      $type = $info['type'];

      if (!isset($context['results']['prepared']) || $context['results']['prepared'] !== TRUE) {
        $context['results']['failed']['copy'][] = ['id' => $id, 'type' => $type];
        return;
      }

      $entity = \Drupal::entityTypeManager()->getStorage($type)->load($id);
      $uris = array_merge($uris, EntityProcessor::processEntity($entity, $options));
    }

    /** @var \Drupal\Core\File\FileSystem $filesystem */
    $filesystem = \Drupal::service('file_system');

    $zip_name = '';

    foreach ($uris as $source) {
      $source_uri = file_uri_target($source);

      $filename = $filesystem->basename($source_uri);
      $dirname = $filesystem->dirname($source_uri);
      $dest_dir = $package_uri . DIRECTORY_SEPARATOR . $dirname;
      $target = $dest_dir . DIRECTORY_SEPARATOR . $filename;

      if (self::copyImage($source, $dest_dir, $target) === FALSE) {
        $context['results']['failed']['copy'][] = $source;
        continue;
      }

      $context['results']['completed_count']++;

      $should_zip = $options['should_zip'];

      if ($should_zip) {
        // Pull the package URI out of the current path to get the relative path to
        // use in the zip file.
        $str = '@' . preg_quote($package_uri, '@') . '@';
        $current_zip_path = preg_replace($str, '', $target);
        $current_real_path = \Drupal::service('file_system')->realpath($target);

        $zip_result = self::addUriToZipAtPath($options['zip_path'], $current_zip_path, $current_real_path);

        if ($zip_result === FALSE) {
          $context['results']['failed']['zip'][] = $target;
          $context['results']['completed_count']--;
        }
      }
    }
  }

  /**
   * Implements callback_batch_operation().
   *
   * Batch operation to duplicate files into the package folder.
   *
   * @param string $packageUri
   *   The URI that the packaging will be happening in.
   * @param array|\ArrayAccess $context
   *   The batch context array.
   */
  private static function copyImage($source, $dest_dir, $target) {
    $result = FALSE;

    if (!file_prepare_directory($dest_dir, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
      \Drupal::logger('content_packager')->error('Failed to create the directory to pack images: %directory', ['%directory' => $dest_dir]);
    }

    if (!file_unmanaged_prepare($source, $target, FILE_EXISTS_REPLACE)) {
      return FALSE;
    }

    $result = file_unmanaged_copy($source, $target, FILE_EXISTS_REPLACE);
    $dt = filemtime($source);
    if ($dt !== FALSE && $result !== FALSE) {
      touch($target, $dt);
    }

    return $result;
  }

  /**
   * Implements callback_batch_operation().
   *
   * @param string $zipfile_path
   *   The full zip file path and filename.
   * @param string $destination_path
   *   The destination path of the file within the zip file's folder structure.
   */
  private static function addUriToZipAtPath($zipfile_path, $destination_path, $full_file_path) {

    // It initially seems like we should be using Drupal's Archiver plugins
    // but they really seem like they do we need, which is Zip creation.
    $zip = new \ZipArchive();
    $opened = $zip->open($zipfile_path, \ZipArchive::CREATE);
    if ($opened === FALSE) {
      return FALSE;
    }

    // ZipArchive operations return true even if file doesn't exist or otherwise
    // has problems, so explicitly detect a failure at this point.
    if (!file_exists($full_file_path) || !is_readable($full_file_path)) {
      return FALSE;
    }

    if ($zip->addFile($full_file_path, $destination_path) === FALSE) {
      return FALSE;
    }
    if ($zip->close() === FALSE) {
      return FALSE;
    }

    return $zipfile_path;
  }

  /**
   * Implements callback_batch_operation().
   */
  public static function zipDataFile($zip_path, $file_uri, $internal_destination_path, &$context) {

    $full_file_path = \Drupal::service('file_system')->realpath($file_uri);

    if (self::addUriToZipAtPath($zip_path, $internal_destination_path, $full_file_path) === FALSE) {
      $context['results']['failed']['zip'][] = $file_uri;
      return;
    }

    $context['results']['completed_count']++;
  }

  /**
   * Implements callback_batch_finished().
   */
  public static function packingFinished($success, $results, $operations) {
    if (!$success) {
      \Drupal::messenger()->addError('The content has not been properly packaged.');
      return;
    }

    $file_count = isset($results['completed_count']) ? $results['completed_count'] : 0;

    \Drupal::messenger()->addStatus(t('Content packaging successfull!'));
    \Drupal::messenger()->addStatus(t('%file_count files copied to content packager directory.  Content packaging successfull!', ['%file_count' => $file_count]));

    if (!empty($results['failed'])) {
      $msg = t(':count files did not copy or get zipped correctly; please refer to the <a href="@system-log">error log</a> for more information.',
        [':count' => count($results['failed']), '@system-log' => './admin/reports/dblog']);
      \Drupal::messenger()->addError($msg);
      return;
    }

  }

}
