<?php

namespace Drupal\content_packager\Form;

use Drupal\Console\Bootstrap\Drupal;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\rest\Plugin\views\display\RestExport;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the content package generation form.
 *
 * @package Drupal\content_packager\Form
 */
class CreatePackage extends FormBase {

  private $entityTypeManager;

  /**
   * Class constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManager $typeManager) {
    $this->entityTypeManager = $typeManager;
    $this->setConfigFactory($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_packager_create_package';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form = [];
    $config = $this->config('content_packager.settings');

    $available_views = content_packager_get_rest_export_views();
    $view_options = [];

    foreach ($available_views as $id => $display) {
      $view_options[$display[0]] = $display[0];
    }

    $selected_view = $form_state->getValue('selected_view', reset($view_options));

    $displays = [];
    if ($selected_view) {
      $view = Views::getView($selected_view);
      $displays = $view->storage->get('display');
    }

    $rest_displays = [];
    foreach ($displays as $display) {
      if ($display['display_plugin'] === 'rest_export') {
        $rest_displays[$display['id']] = $display['id'];
      }
    }

    $package_uri = content_packager_package_uri();
    $zip_name = $config->get('zip_name');
    $this->buildExistingPackageInfo($package_uri, $zip_name, $form);

    $form['make_package'] = [
      '#type' => 'details',
      '#title' => t('Make a content package'),
      '#open' => TRUE,
    ];
    $form['make_package']['selected_view'] = [
      '#type' => 'select',
      '#title' => t('View to export'),
      '#default_value' => '',
      '#options' => $view_options,
      '#ajax' => [
        'callback' => [$this, 'viewSelectChanged'],
        'event' => 'change',
        'wrapper' => 'display-wrapper',
      ],
    ];

    $form['make_package']['display_ajax'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => ['display-wrapper'],
      ],
    ];

    $form['make_package']['display_ajax']['selected_display'] = [
      '#type' => 'select',
      '#title' => t('View display'),
      '#default_value' => '',
      '#options' => $rest_displays,
    ];

    $form['make_package']['destination'] = [
      '#markup' => "<p>Package destination: $package_uri</p>",
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Make a new package'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  public function viewSelectChanged(array $form, FormStateInterface $form_state) {
    return $form['make_package']['display_ajax'];
  }

  private function buildExistingPackageInfo($package_uri, $zip_name, &$form) {
    $full_package_uri = $package_uri . DIRECTORY_SEPARATOR . $zip_name;
    if (!file_exists($full_package_uri)) {
      return;
    }

    $url = file_create_url($full_package_uri);

    $form['existing_file'] = [
      '#type' => 'details',
      '#title' => t('Current Package'),
      '#prefix' => t('A package already exists and <a href=":package_link">can be downloaded</a> at any time.',
        [':package_link' => $url]),
      '#open' => FALSE,
    ];

    $form['existing_file']['file'] = [
      '#type' => 'item',
      '#title' => 'File',
      '#markup' => t('<a href=":package_uri">%display_uri</a>',
        [
          ':package_uri' => $url,
          '%display_uri' => $full_package_uri,
        ]),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // Make sure our directory is actually there!
    $package_uri = content_packager_package_uri();

    if (!file_exists($package_uri)) {
      $errors = content_packager_prepare_directory($package_uri);

      if (!file_exists($package_uri)) {
        $msg = t('Directory for packaging is incorrectly configured or cannot be found.');
        \Drupal::messenger()->addError($msg);
        return;
      }
    }

    $batch_size = $this->config('content_packager.settings')->get('batch_size');
    $should_zip = $this->config('content_packager.settings')->get('create_zip');
    $image_styles = array_keys($this->config('content_packager.settings')->get('image_styles'));
    $field_blacklist = $this->config('content_packager.settings')->get('fields_ignored');
    $zip_name = $this->config('content_packager.settings')->get('zip_name');
    $zipfile_path = \Drupal::service('file_system')->realpath($package_uri);
    $zipfile_path .= DIRECTORY_SEPARATOR . $zip_name;

    $view_id = $form_state->getValue('selected_view');
    $display_id = $form_state->getValue('selected_display');
    $view = Views::getView($view_id);
    $view->setDisplay($display_id);

    $has_access = $view->access($display_id);
    $display = $view->getDisplay();

    $content_filename = $display instanceof RestExport ?
      "{$display_id}.{$display->getContentType()}" : "{$display_id}.data";

    $data_uri = $package_uri . DIRECTORY_SEPARATOR . $content_filename;

    $operations[] = [
      'Drupal\content_packager\BatchOperations::prepareDestination',
      [$package_uri],
    ];

    $options = [
      'image_styles' => $image_styles,
      'field_blacklist' => $field_blacklist,
      'data_path' => $data_uri,
      'should_zip' => $should_zip,
    ];

    if ($should_zip) {
      $options['zip_name'] = $zip_name;
      $options['zip_path'] = $zipfile_path;
    }

    if (!$display_id) {
      $context['results']['rendered'] = FALSE;
      $msg = t('Could not find an appropriate REST Export display for the selected view to render!');
      \Drupal::messenger()->addError($msg);
      return;
    }

    if (!$has_access) {
      $operations[] = [
        'Drupal\content_packager\BatchOperations::renderAndSaveViewOutput',
        [$view_id, $display_id, $data_uri],
      ];
    }

    // This function does a lot, so we aren't using the existing $view.
    $result = $has_access ? views_get_view_result($view_id, $display_id) : [];
    $result_chunks = array_chunk($result, $batch_size);

    foreach ($result_chunks as $results) {
      $infos = [];
      foreach ($results as $result) {
        /* @var \Drupal\Core\Entity\Entity $entity */
        $entity = $result->_entity;

        $access = $entity->access('view');
        if (!$access) {
          continue;
        }
        $infos[] = ['id' => $entity->id(), 'type' => $entity->getEntityTypeId()];
      }

      $operations[] = [
        'Drupal\content_packager\BatchOperations::copyEntityFilesAndZip',
        [
          $infos,
          $package_uri,
          $options,
        ],
      ];
    }

    if ($should_zip) {
      $operations[] = [
        'Drupal\content_packager\BatchOperations::zipDataFile',
        [$zipfile_path, $data_uri, $content_filename],
      ];
    }

    $batch = [
      'operations' => $operations,
      'finished' => 'Drupal\content_packager\BatchOperations::packingFinished',
    ];

    batch_set($batch);
  }

}
